#!/usr/bin/env python
##
##  mp3add.py - adding mp3 file to a FLV movie.
##
##  Copyright (c) 2009 by Yusuke Shinyama
##

import sys, os, re
from vnc2flv.flv import FLVParser, FLVWriter
from vnc2flv.audio import FLVAudioSink
from vnc2flv.video import MultipleRange


##  mp3add
##
def mp3add(srcfile, mp3files, outfile, force=False, debug=0):
  if not force:
    try:
      os.stat(outfile)
      raise IOError('file already exists: %r' % outfile)
    except OSError:
      pass
  fout = file(outfile, 'wb')
  writer = FLVWriter(fout, debug=debug, has_video=True, has_audio=True)
  audiosink = FLVAudioSink(writer)
  for fname in mp3files:
    ranges = None
    m = re.match(r'([^:]+):(.+)$', fname)
    if m:
      fname = m.group(1)
      ranges = MultipleRange(m.group(2))
    fp = file(fname, 'rb')
    audiosink.add_mp3(fp, timeranges=ranges)
    fp.close()
  fin = file(srcfile, 'rb')
  parser = FLVParser(fin, debug=debug)
  for (i, (tag, _, timestamp, _)) in enumerate(parser):
    if tag == 8: continue
    audiosink.flush(timestamp)
    writer.start_tag()
    writer.write(parser.get_data(i))
    writer.end_tag(tag, timestamp)
  parser.close()
  writer.close()
  fin.close()
  fout.close()
  return


# main
def main(argv):
  import getopt, vnc2flv
  def usage():
    print argv[0], vnc2flv.__version__
    print ('usage: %s [-d] [-f] fin [mp3:range ...] fout' % argv[0])
    return 100
  try:
    (opts, args) = getopt.getopt(argv[1:], 'df')
  except getopt.GetoptError:
    return usage()
  debug = 0
  force = False
  for (k, v) in opts:
    if k == '-d': debug += 1
    elif k == '-f': force = True
  if len(args) < 3: return usage()
  srcfile = args.pop(0)
  outfile = args.pop(-1)
  mp3add(srcfile, args, outfile, force=force, debug=debug)
  return

if __name__ == "__main__": sys.exit(main(sys.argv))
